.if t .ds LX L\\h'-0.36m'\\v'-0.15v'\\s-2A\\s+2\\h'-0.15m'\\v'0.15v'T\\h'-0.1667m'\\v'0.20v'E\\v'-0.20v'\\h'-0.125m'X
.if t .ds TX T\\h'-0.1667m'\\v'0.20v'E\\v'-0.20v'\\h'-0.125m'X
.if n .ds TX TeX
.if n .ds LX LaTeX
.TH MAKEINDEX 1L "11/11/1989"
.SH NAME
makeindex \- a general purpose, formatter-independent index processor
.SH SYNOPSIS
\fBmakeindex\fR [\fB\-ilqrcg\fR] [\fB-s\fI sty\fR] [\fB-o\fI ind\fR]
[\fB-t\fI log\fR] [\fB-p\fI no\fR] [\fI idx0 idx1 idx2 ...\fR]
.SH DESCRIPTION
.I Makeindex
is a general purpose hierarchical index generator; it accepts one or more
input files (often produced by a text formatter such as \*(TX
.RI ( tex (1L))
or
.IR troff (1)),
sorts the entries, and produces an output file which can be formatted.
The index can have up to three levels (0, 1, and 2) of subitem nesting.
The way in which words are flagged for indexing within the main document
is specific to the formatter used; this program does \fInot\fR automate
the process of selecting these words.
As the output index is hierarchical, \fI makeindex\fR can be considered
complimentary to the \fI awk\fR (1)-based \fImake.index\fR (1L)
system of Bentley and Kernighan, which is specific to \fI troff\fR (1),
generates non-hierarchical indices, and employs a much
simpler syntax for indicating index entries.
.LP
The formats of the input and output files are
specified in a style file; by default, input is assumed to be a \fI.idx\fR
file, as generated by \*(LX.
.LP
Unless specified explicitly, the base name of the first input file
(\fIidx0\fR) is used to determine the names of other files.
For each input file name specified, a file of that name is sought.  If this
file is not found and the file name has no extension, the extension \fI.idx\fR
is appended.  If no file with this name is found, the program aborts.
.LP
For important notes on how to select index keywords, see the document by
Lamport cited below.  As an issue separate from selecting index keywords,
a systematic mechanism for placing index terms in a document
is suggested in ``\fIIndex Preparation and Processing\fR'', a paper
cited below.
.SH OPTIONS
.TP 10
\fB\-i\fR	
Take input from \fIstdin\fR.  When this option is specified and
\fB\-o\fR is not, output is written to \fIstdout\fR.
.TP 10
\fB\-l\fR
Letter ordering; by default, word ordering is used (see the \fBORDERING\fR
section).
.TP 10
\fB\-q\fR
Quiet mode; send no messages to \fIstderr\fR.  By default, progress and
error messages are sent to \fIstderr\fR as well as to the transcript file.
.TP 10
\fB\-r\fR
Disable implicit page range formation; page ranges must be created by
using explicit range operators (see \fBSPECIAL EFFECTS\fR).
By default, three or more successive pages are automatically abbreviated as
a range (e.g. 1\(em5).
.TP 10
\fB\-c\fR
Compress intermediate blanks (ignoring leading and trailing blanks and tabs).
By default, blanks in the index key are retained.
.TP 10
\fB\-g\fR
By default \fImakeindex\fR's word ordering is that symbols precede
numbers which precede uppercase letters which precede lowercase
letters.  The sequence in German word ordering is symbols, lowercase
letters, uppercase letters, then numbers.  By setting the new option
\fImakeindex\fR sorts the entries considering that word ordering
according to the German rule DIN 5007.  Moreover the \fB-g\fR option
enables \fImakeinde\fR to recognize the German \*(TX-commands {"a, "o,
"u and "s} as {ae, oe, ue and ss} during the sorting of the entries.
If German sort is active, the quote character must be redefined in a
style file, e.g., redefine quote as '+'.  If \fB-g\fR is set and the
quote character is not redefined \fImakeindex\fR will produce an error
message and abort.
.TP 10
\fB\-s\fI sty\fR
Employ \fIsty\fR as the style file (no default).  The environment variable
INDEXSTYLE defines the path where the style file should be found.
.TP 10
\fB\-o\fI ind\fR
Employ \fIind\fR as the output index file.  By default, the file name is
created by appending the extension \fI.ind\fR to the base name of the first
input file (\fIidx0\fR).
.TP 10
\fB\-t\fI log\fR
Employ \fIlog\fR as the transcript file.  By default, the file name is
created by appending the extension \fI.ilg\fR to the base name of the first
input file (\fIidx0\fR).
.TP 10
\fB\-p\fI num\fR
Set the starting page number of the output index file to be \fInum\fR
(useful when the index file is to be formatted separately).  The argument
\fInum\fP may be numerical or one of the following:
.RS
.TP 10
\fIany\fR
The starting page is the last source page number plus 1.
.TP 10
\fIodd\fR
The starting page is the first odd page following the last source page number.
.TP 10
\fIeven\fR
The starting page is the first even page following the last source page number.
.RE
.IP
The source log file name is determined by appending the extension \fI.log\fR
to the base name of the first input file (\fIidx0\fR).  The last source page
is obtained by searching backward in the log file for the first instance of
a number included within paired square brackets (\fB[\fR...\fB]\fR).
If a page number is missing or the log file is not found, no attempt will be
made to set the starting page number.
.SH "STYLE FILE"
This file can reside anywhere in the path defined by the environment
variable INDEXSTYLE.
A style file is a list of <\fIspecifier\fR, \fIattribute\fR> pairs.
There are two types of specifiers: input and output.
Pairs do not have to appear in any particular order.
A line begun by `%' is a comment.  In the following list of
specifiers and arguments, <string> is an arbitrary string delimited by
double quotes ("..."), <char> is a single letter embraced by
single quotes ('...'), and <number> is a nonnegative integer.
The maximum length of a <string> is 1024.  A literal backslash or quote
must be escaped (by a backslash).  Anything not specified in the style file
will be assigned a default value, which is shown in the rightmost
column.
.SS 
Input Style Specifiers
.TP 25
\fBkeyword\fR  <string>
"\\\\indexentry"
.RS
Command which tells \fImakeindex\fR that its argument is
an index entry. 
.RE
.TP
\fBarg_open\fR  <char>
\'{\'
.RS
Opening delimiter for the index entry argument.
.RE
.TP
\fBarg_close\fR  <char>
\'}\'
.RS
Closing delimiter for the index entry argument.
.RE
.TP
\fBrange_open\fR  <char>
\'(\'
.RS
Opening delimiter indicating the beginning of an explicit page range.
.RE
.TP
\fBrange_close\fR  <char>
\')\'
.RS
Closing delimiter indicating the end of an explicit page range.
.RE
.TP
\fBlevel\fR  <char>
\'!\'
.RS
Delimiter denoting a new level of subitem.
.RE
.TP
\fBactual\fR  <char>
\'@\'
.RS
Symbol indicating that the next entry is to appear in the output file.
.RE
.TP
\fBencap\fR  <char>
\'|\'
.RS
Symbol indicating that the rest of the argument list
is to be used as the encapsulating command for the page number.
.RE
.TP
\fBquote\fR  <char>
\'"\'
.TP
\fBescape\fR  <char>
\'\\\\\'
.RS
Symbol which escapes the following letter, unless its preceding letter
is \fBescape\fR.  Note: \fBquote\fR is used to escape the letter
which immediately follows it, but if it is preceded by \fBescape\fR,
it is treated as a ordinary character.  These two symbols
.I must
be distinct.
.RE
.sp 1
.SS
Output Style Specifiers
.TP 25
\fBpreamble\fR  <string>
"\\\\begin{theindex}\\n"
.RS
Preamble of output file.
.RE
.TP
\fBpostamble\fR  <string>
"\\n\\n\\\\end{theindex}\\n"
.RS
Postamble of output file.
.RE
.TP
\fBsetpage_prefix\fR  <string>
"\\n  \\\\setcounter{page}{"
.RS
Prefix of command which sets the starting page number.
.RE
.TP
\fBsetpage_suffix\fR  <string>
"}\\n"
.RS
Suffix of command which sets the starting page number.
.RE
.TP
\fBgroup_skip\fR  <string>
"\\n\\n  \\\\indexspace\\n"
.RS
Vertical space to be inserted before a new group begins.
.RE
.TP
\fBheadings_flag\fR  <string>
0
.RS
Flag indicating treatment of new group headers, which are inserted when
before a new group (symbols, numbers, and the 26 letters): positive values
cause an uppercase letter to be inserted between prefix and suffix, and
negative values cause a lowercase letter to be inserted (default is 0,
which produces no header).
.RE
.TP
\fBheading_prefix\fR  <string>
""
.RS
Header prefix to be inserted before a new letter begins.
.RE
.TP
\fBsymhead_positive\fR  <string>
"Symbols"
.RS
Heading for symbols to be inserted if \fBheadings_flag\fR is positive.
.RE
.TP
\fBsymhead_negative\fR  <string>
"symbols"
.RS
Heading for symbols to be inserted if \fBheadings_flag\fR is negative.
.RE
.TP
\fBnumhead_positive\fR  <string>
"Numbers"
.RS
Heading for numbers to be inserted if \fBheadings_flag\fR is positive.
.RE
.TP
\fBnumhead_negative\fR  <string>
"numbers"
.RS
Heading for numbers to be inserted if \fBheadings_flag\fR is negative.
.RE
.TP
\fBitem_0\fR  <string>
"\\n  \\\\item "
.RS
Command to be inserted between two primary (level 0) items.
.RE
.TP
\fBitem_1\fR  <string>
"\\n     \\\\subitem "
.RS
Command to be inserted between two secondary (level 1) items.
.RE
.TP
\fBitem_2\fR  <string>
"\\n       \\\\subsubitem "
.RS
Command to be inserted between two level 2 items.
.RE
.TP
\fBitem_01\fR  <string>
"\\n    \\\\subitem "
.RS
Command to be inserted between a level 0 item and a level 1 item.
.RE
.TP
\fBitem_x1\fR  <string>
"\\n    \\\\subitem "
.RS
Command to be inserted between a level 0 item and a level 1 item,
where the level 0 item does not have associated page numbers.
.RE
.TP
\fBitem_12\fR  <string>
"\\n    \\\\subsubitem "
.RS
Command to be inserted between a level 1 item and a level 2 item.
.RE
.TP
\fBitem_x2\fR  <string>
"\\n    \\\\subsubitem "
.RS
Command to be inserted between a level 1 item and a level 2 item,
where the level 1 item does not have associated page numbers.
.RE
.TP
\fBdelim_0\fR  <string>
", "
.RS
Delimiter to be inserted between a level 0 key and its first page number
(default: comma followed by a blank).
.RE
.TP
\fBdelim_1\fR  <string>
", "
.RS
Delimiter to be inserted between a level 1 key and its first page number
(default: comma followed by a blank).
.RE
.TP
\fBdelim_2\fR  <string>
", "
.RS
Delimiter to be inserted between a level 2 key and its first page number
(default: comma followed by a blank).
.RE
.TP
\fBdelim_n\fR  <string>
", "
.RS
Delimiter to be inserted between two page numbers for the same key
in any level (default: comma followed by a blank).
.RE
.TP
\fBdelim_r\fR  <string>
"--"
.RS
Delimiter to be inserted between the starting and ending page numbers
of a range.
.RE
.TP
\fBdelim_t\fR  <string>
""
.RS
Delimiter to be inserted at the end of a page list.  This delimeter
has no effect on entries which have no associated page list.
.RE
.TP
\fBencap_prefix\fR  <string>
"\\\\"
.RS
First part of prefix for the command which encapsulates the page number.
.RE
.TP
\fBencap_infix\fR  <string>
"{"
.RS
Second part of prefix for the command which encapsulates the page number.
.RE
.TP
\fBencap_suffix\fR  <string>
"}".
.RS
Suffix for the command which encapsulates the page number.
.RE
.TP
\fBline_max\fR  <number>
72
.RS
Maximum length of a line in the output, beyond which a line wraps.
.RE
.TP
\fBindent_space\fR  <string>
"\\t\\t"
.RS
Space to be inserted in front of a wrapped line (default: two tabs).
.RE
.TP
\fBindent_length\fR  <number>
16
.RS
Length of \fBindent_space\fR (default: 16, equivalent to 2 tabs).
.RE
.SH "\*(TX EXAMPLE"
The following example shows a style file called \fIbook.ist\fR, which
defines an index for a book which can be formatted independently of the
main source:
.LP
.RS
.nf
\fCpreamble
"\\\\documentstyle[12pt]{book}
\\\\begin{document}
\\\\begin{theindex}
{\\\\small\\n"
postamble
"\\n\\n}
\\\\end{theindex}
\\\\end{document}\\n\fP"
.fi
.RE
.LP
Assuming that a particular book style requires the index (as well as any
chapters) to start from an odd page number, and that the input file is
named \fIfoo.idx\fR, the following command line produces output in
file \fIfoo-.ind\fR:
.LP
.RS
\fCmakeindex  \-s book.ist  \-o foo-.ind  \-p odd  foo\fP
.RE
.LP
Here a non-default output file name is used to avoid clobbering the
output for the book itself (presumably \fIfoo.dvi\fR, which would have been
the default name for the index output file!).
.SH "TROFF EXAMPLE"
A sample control file for creating an index, which we will assume resides in
the file
.IR sample.ist :
.LP
.RS
.nf
\&\fCkeyword "IX:"
\&preamble
\&".\\\\\\" start of index output
\&.SH
\&.ce
\&\\\\s+2INDEX\\\\s-2
\&.2C
\&.de IS
\&.sp 0.5
\&\\\\s+2\\\\fB
\&.ce
\&..
\&.de IE
\&.sp 0.5
\&\\\\s-2\\\\fP
\&..
\&.de I1
\&.ti 0.25i
\&..
\&.de 1I
\&.ti 0.25i
\&..
\&.de I2
\&.ti 0.5i
\&..
\&.de 2I
\&.ti 0.5i
\&.."
\&postamble "\\n.\\\\\\" end of index output"
\&setpage_prefix "\\n.nr % 1\\n"
\&setpage_suffix ""
\&group_skip "\\n.sp"
\&headings_flag 1
\&heading_prefix "\\n.IS\\n"
\&heading_suffix "\\n.IE"
\&item_0 "\\n.br\\n"
\&item_1 "\\n.I1\\n"
\&item_2 "\\n.I2\\n"
\&item_01 "\\n.I1\\n"
\&item_x1 "\\n.1I\\n"
\&item_12 "\\n.I2\\n"
\&item_x2 "\\n.2I\\n"
\&delim_0 ", "
\&delim_1 ", "
\&delim_2 ", "
\&delim_r "-"
\&delim_t "."
\&encap_prefix "\\\\fB"
\&encap_infix ""
\&encap_suffix "\\\\fP"
\&indent_space "	"
\&indent_length 8\\fP
.fi
.RE
.LP
The local macro package may require modification, as in this example of an
extension to the
.B \-ms
macros (note that at some sites, this macro should
.I replace
a pre-existing macro of the same name):
.LP
.RS
.nf
\fC\&.\" IX - index words to stderr
\&.de IX
\&.ie '\\\\n(.z'' .tm IX: \\\\$1 \\\\$2 \\\\$3 \\\\$4 \\\\$5 \\\\$6 \\\\$7 \\\\$8 \\\\$9 {\\\\n(PN}
\&.el \\\\!.IX \\\\$1 \\\\$2 \\\\$3 \\\\$4 \\\\$5 \\\\$6 \\\\$7 \\\\$8 \\\\$9 {\\\\n(PN}
\&..\fP
.fi
.RE
.LP
(note that the string {\\\\n(PN} is separated from the rest of the line by
a tab.  If your local macro package does not contain this extension,
just include those lines at the beginning of your file.
Here is a simple
.IR troff (1)
input file, which we will assume is named
.IR sample.txt :
.LP
.RS
.nf
\fC\&This is a sample file to test the \\fImakeindex\\fP(1L) program, and see
\&.IX {indexing!programs!C language}
\&.IX {makeindex@\\fImakeindex\\fP(1L)}
\&.bp
\&.rs
\&.IX {Knuth}
\&.IX {typesetting!computer-aided}
\&how well if functions in the \\fItroff\\fP(1) environment.\fP
.fi
.RE
.LP
Note that index entries are indicated by the \fB.IX\fP macro, which
causes the following text to be written to \fIstdout\fP along with the current
page number.  To create an input file for
.IR makeindex ,
\fBin the Bourne shell\fP environment, do the
equivalent at your site of the command:
.LP
.RS
\fCpsroff -ms -Tpsc -t sample.txt > /dev/null 2> sample.tmp\fP
.RE
.LP
To filter out any genuine error messages, invoke
.IR grep (1):
.LP
.RS
\fCgrep '^IX: ' sample.tmp > sample.idx\fP
.RE
This leaves the input for
.I makeindex
in
.IR sample.inp ;
the next step is to invoke makeindex:
.LP
.RS
\fCmakeindex -s sample.ist sample.idx\fP
.RE
.LP
which leaves
.IR troff (1)-ready
output in the file
.IR sample.ind .
.SH "ORDERING"
By default, \fImakeindex\fR assumes \fIword ordering\fR;  if the \fB\-l\fR
option is in effect, \fIletter ordering\fR is used.  In word ordering,
a blank precedes any letter in the alphabet, whereas in letter ordering,
it does not count at all.  This is illustrated by the following example:
.LP
.RS
\fIword order			letter order\fR
.br
sea lion				seal
.br
seal				sea lion
.RE
.LP
Numbers are always sorted in numeric order.  For instance,
.LP
.RS
9 (nine),  123
.br
10 (ten), see Derek, Bo
.RE
.LP
Letters are first sorted without regard to case; when words are identical,
the uppercase version precedes its lowercase counterpart.
.LP
A special symbol is defined here to be any character not appearing in the
union of digits and the English alphabetic.  Patterns starting with special
symbols precede numbers, which precede patterns starting with letters.
As a special case, a string starting with a digit but mixed with
non-digits is considered to be a pattern starting with a special character.
.SH "SPECIAL EFFECTS"
Entries such as
.LP
.RS
.nf
\fC\\indexentry{alpha}{1}
\\indexentry{alpha!beta}{3}
\\indexentry{alpha!beta!gamma}{10}\fP
.fi
.RE
in the input file will be converted to
.LP
.RS
.nf
\fC\\item alpha, 1
\0\0\0\\subitem beta, 3
\0\0\0\0\0\0\\subsubitem gamma, 10\fP
.fi
.RE
in the output index file.
Notice that the \fBlevel\fR symbol (`!') is used above to delimit
hierarchical levels.
.LP
It is possible to make an item appear in a designated form by
using the \fBactual\fR (`@') operator.  For instance,
.LP
.RS
\fC\\indexentry{alpha@{\\it alpha\\/}}{1}\fP
.RE
will become
.LP
.RS
\fC\\item {\\it alpha\\/}  1\fP
.RE
after processing.  The pattern preceding `@' is
used as sort key, whereas the one following it is written to the output
file.  Note that two appearances of the same key, one with and one without
the \fBactual\fR operator, are regarded as
.I distinct
entries.
.LP
It is possible to encapsulate a page number with a designated
command using the \fBencap\fR (`|') operator:
.LP
.RS
\fC\\indexentry{alpha|bold}{1}\fP
.RE
will be converted to 
.LP
.RS
\fC\\item alpha \\bold{1}\fP
.RE
where \\bold{n} will expand to {\\bf n}.
In this example, the three output attributes associated with page
encapsulation \fBencap_prefix\fR, \fBencap_infix\fR, and \fBencap_suffix\fR,
correspond to backslash, left brace, and right brace, respectively.
This mechanism allows page numbers to be set
in different fonts.  For example, the page where the definition of a keyword
appears can be in one font, the location of a primary example
can be in another font, and other appearances in yet a third font.
.LP
The \fBencap\fR operator can also be used to create cross references in
the index:
.LP
.RS
\fC\\indexentry{alpha|see{beta}}{1}\fP
.RE
will become
.LP
.RS
\fC\\item alpha \\see{beta}{1}\fP
.RE
in the output file, where
.LP
.RS
\fC\\see{beta}{1}\fP
.RE
will expand to
.LP
.RS
\fC{\\it see\\/} beta\fP
.RE
Note that in a cross reference like this the page number disappears.
.LP
A pair of \fBencap\fR concatenated with \fBrange_open\fR (`|(') and
\fBrange_close\fR (`|)') creates an explicit page range:
.LP
.RS
.nf
\fC\\indexentry{alpha|(}{1}
\\indexentry{alpha|)}{5}\fP
.fi
.RE
will become
.LP
.RS
\fC\\item alpha, 1\(em5\fP
.RE
Intermediate pages indexed by the same key will be merged into the range
implicitly.  This is especially useful when an entire section about
a particular subject is to be indexed, in which case only the range opening and
closing operators need to be inserted at the beginning and end of the
section.  Explicit page range formation can also include an extra command
to set the page range in a designated font:
.LP
.RS
.nf
\fC\\indexentry{alpha|(bold}{1}
\\indexentry{alpha|)}{5}\fP
.fi
.RE
will become
.LP
.RS
\fC\\item alpha, \\bold{1--5}\fP
.RE
Several potential problems are worth mentioning.  First, entries like
.LP
.RS
.nf
\fC\\indexentry{alpha|(}{1}
\\indexentry{alpha|bold}{3}
\\indexentry{alpha|)}{5}\fP
.fi
.RE
will be interpreted as
.LP
.RS
\fC\\item alpha, \\bold{3}, 1--5\fP
.RE
but with a warning message in the transcript about encountering an
inconsistent page encapsulator.
An explicit range beginning in a Roman page number and ending
in Arabic is also considered an error.  In this instance, (if possible)
the range is broken into two subranges, one in Roman and the other in Arabic.
For instance,
.LP
.RS
.nf
\fC\\indexentry{alpha|(}{i}
\\indexentry{alpha}{iv}
\\indexentry{alpha}{3}
\\indexentry{alpha|)}{7}\fP
.fi
.RE
will be turned into
.LP
.RS
\fC\\item alpha, 1--iv, 3--7\fP
.RE
with a warning message in the transcript file complaining about an illegal
range formation.
.LP
Finally, every special symbol mentioned in this section may be
escaped by the \fBquote\fR operator (`"').  Thus
.LP
.RS
\fC\\indexentry{alpha"@beta}{1}\fP
.RE
will actually become
.LP
.RS
\fC\\item alpha@beta,  1\fP
.RE
as a result of executing \fImakeindex\fR.  The quoting power of \fBquote\fR is
eliminated if it is immediately preceded by \fBescape\fR (`\\').
For example,
.LP
.RS
\fC\\indexentry{f\\"ur}{1}\fP
.RE
becomes 
.LP
.RS
\fC\\item f\\"ur, 1\fP
.RE
which represents an umlaut-accented `u' to the \*(TX family of processors.
.SH "SEE ALSO"
latex(1L), make.index (1L), qsort(3), tex(1L), troff(1L)
.br
\fIIndex Preparation and Processing\fP, Pehong Chen and Michael A. Harrison.
\fISoftware: Practice and Experience\fR, 18(9):897-915, September 1988.
.br
\fIAutomating Index Preparation\fP, Pehong Chen and Michael A. Harrison.
Technical Report 87/347,
Computer Science Division, University of California, Berkeley, 1988.
This is a \*(LX document supplied with the \fImakeindex\fP distribution.
.br
\fIMakeIndex: An Index Processor for \*(LX\fP, Leslie Lamport,
Feburary 1987.  This is a \*(LX document supplied with the \fImakeindex\fP
disstribution.
.br
\fITools for Printing Indices\fP, Jon L. Bentley and Brian W. Kernighan,
\fIElextronic Publishing\fR, 1(1), June 1988.  Also available as
Computing Science Technical Report No. 128, AT&T Bell Laboratories,
Murray Hill, NJ 07974, 1986.
.SH AUTHOR
Pehong Chen, Chen & Harrison International Systems, Inc.
Palo Alto, California, USA (chen@renoir.berkeley.edu).
.br
Manual page extensively revised and corrected, and
.IR troff (1)
examples created by Rick P. C. Rodgers, UCSF School
of Pharmacy (rodgers@cca.ucsf.edu).
.SH ACKNOWLEDGMENTS
Leslie Lamport contributed significantly to the design.
Michael Harrison provided valuable comments and suggestions.
Nelson Beebe improved on the portable version significantly.
Andreas Brosig contributed to the German word ordering.
The modification to the
.B \-ms
macros was derived from a method proposed by Ravi Sethi of AT&T
Bell Laboratories.
